import greenfoot.*;  // (Actor, World, Greenfoot, GreenfootImage).
import java.util.*; // In particular, this import will allow me to utilize lists.
import java.lang.*; // In particular, this import will allow me to use Intger.parseInt(java.lang.string);

/**
 * This class defines a Wilderness. This is the world where Animals roam and
 * Vegetation grows.
 * 
 * @author Isaac Chen 
 * @version Version 1, February 2023
 */
public class Wilderness extends World 
{    
    int[] xAnimals;
    int[] yAnimals;
    int[] xVegetation = new int[50];
    int[] yVegetation = new int[50];

    public static int initialPopulation;
    public static int currentPopulation;
    public static int totalVegetationEaten;
    public static int worldTime;
    public static int scaleUp;
   
    /** 
     * Constructor creates the Wilderness which is 900x600 cells, where every cell is 
     * just 1 pixel.
     * No return type is specified.
     * A constructor has the same name as the class.
     * A constructor is automatically executed whenever an instance of the class is created.
     * This constructor displays the time left, total vegetation consumed, and current population 
     * of the Wilderness.
     * A number of Animals (inputed by the user) and 50 bushes (from the Vegetation class) are also 
     * initialized with random locations (see the arrays).
     */
    public Wilderness()
    {
        super(900, 600, 1, true); 
        // The term "super" allows us to access the constructor of this class' superclass, which is just World.        
        setPaintOrder(Animal.class, Vegetation.class);
        
        initialPopulation = Integer.parseInt(Greenfoot.ask("How many animals would you like to start in the world? Start with around 30 animals; feel free to experiment with different starting values afterwards!"));
                
        xAnimals = new int[initialPopulation];
        yAnimals = new int[initialPopulation];

        totalVegetationEaten = 0;
        worldTime = 900;

        scaleUp = Integer.parseInt(Greenfoot.ask("If you enter the positive integer x, then each animal's lifespan will range from x to x + 99 (inclusive). Start by trying x = 300; feel free to experiment with different starting values afterwards!"));
        
        prepare();
        currentPopulation = initialPopulation;
        
        showTime();
        showConsumed();
        showPopulation();
    }
    
    /**
     * First checks if-condtion to see if it's time to add a Vegetation
     * in a random location within the world.
     * Then, the act method updates the "Time Left", "Total Vegetation Eaten",
     * and "Current Population" displays.
     * Finally, it checks if it's time to wipeout a certain region of the 
     * Wilderness (by running bigWipeout).
     */
    public void act()
    {
        if(Greenfoot.getRandomNumber(15) == 0)
        {
            addObject(new Vegetation(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        }
        
        countTime();
        showConsumed();
        showPopulation();
        bigWipeout();
    }
    
    /**
     * Decreases the worldTime variable by 1 each time it's run and redisplays 
     * the correct time. If worldTime is 0, the simulation ends and a sound 
     * is played.
     */
    private void countTime()
    {
        worldTime--;
        showTime();
        
        if(worldTime == 0)
        {
            Greenfoot.stop();
            Greenfoot.playSound("fanfare.wav");
        }
    }
    
    /**
     * This method displays the current time left in the simulation by using the
     * worldTime static variable.
     */
    private void showTime()
    {
        showText("Time Left: " + worldTime, 70, 20);
    }
    
    /**
     * This method displays the total number of bushes eaten by Animals so far
     * during the simulation by calling the totalVegetationEaten static variable
     * from the Animal class.
     */
    private void showConsumed()
    {
        showText("Total Vegetation Eaten: " + totalVegetationEaten, getWidth() - 130, 20);
    }
    
    /**
     * This method displays the current Animal population in the world by using the
     * currentPopulation static variable.
     */
    private void showPopulation()
    {
        showText("Current Animal Population: " + currentPopulation, getWidth() - 170, getHeight() - 20);
    }
    
    /**
     * Prepare the world for the start of the program.
     * That is: create the initial objects and add them to the world.
     * 
     * This method first calls getcoordinates() to fill the four arrays
     * seen below. It then uses a for and while loop, using two distinct 
     * pairs of these four arrays to add 30 Animals and 30 Vegetations 
     * with (random) coordinates from said arrays.
     */
    private void prepare()
    {
        getCoordinates();

        for(int i = 0; i < initialPopulation; i++)
        {
            // System.out.println(i);
            // System.out.println(currentPopulation);
            addObject(new Animal(), xAnimals[i], yAnimals[i]);
        }

        int j = 0; 

        while(j < 50)
        {
            addObject(new Vegetation(), xVegetation[j], yVegetation[j]);
            j++;
        }
    }
    
    /**
     * This method fills the four arrays seen below with random integers which will 
     * be the coordinates of new Animal and Vegetation objects. The world methods
     * getWidth() and getHeight() are used to provide the correct bounds, regardless
     * of the actual Wilderness dimensions.
     */
    private void getCoordinates()
    {
        for(int i = 0; i < initialPopulation; i++)
        {
            xAnimals[i] = Greenfoot.getRandomNumber(getWidth());
            yAnimals[i] = Greenfoot.getRandomNumber(getHeight());
        }
        
        int j = 0;
        
        while(j < 50)
        {
            xVegetation[j] = Greenfoot.getRandomNumber(getWidth());
            yVegetation[j] = Greenfoot.getRandomNumber(getHeight());
            j++;
        }
    }
    
    /**
     * The purpose of this method is to allow for the possibilty of a natural disaster
     * (think asteroid hitting the region) wiping out a significant portion of the 
     * Animal population. If the boolean method checkIfDestroy returns true, then 
     * the method picks a random center of disaster. Then, it utlizes a list to find 
     * all Animals within 150 units wrt x-distance and y-distance of the disaster center, 
     * removing all satisfactory animals.
     */
    private void bigWipeout()
    {
        if(checkIfDestroy())
        {
            int xCenter = Greenfoot.getRandomNumber(getWidth() - 300) + 150;
            int yCenter = Greenfoot.getRandomNumber(getHeight() - 300) + 150;
            
            List<Animal> animals = getObjects(Animal.class);
            
            for(Animal animal : animals)
            {
                int displacementX = animal.getX() - xCenter;
                int displacementY = animal.getY() - yCenter;

                if((displacementX < 150) && (displacementX > -150) && (displacementY < 150) && (displacementY > -150))
                {
                    removeObject(animal);
                }
            }
            
            Greenfoot.playSound("explosion.wav");
        }
    }
    
    /**
     * Returns true or false; helps to decide if a wipeout will occur
     * via bigWipeout (see immediately above).
     */
    private boolean checkIfDestroy()
    {
        if(Greenfoot.getRandomNumber(400) < 1)
        {
            return true;
        }
        
        else
        {
            return false;
        }
    }
}